@@ -1,21 +1,57 @@
+import json
+import jwt
+import requests
+import re
 import os
-from datetime import date, datetime, time, timedelta
-from dotenv import load_dotenv
-from url_remote.environment_name_enum import EnvironmentName
+import random
+from datetime import date, datetime, time, timedelta, timezone
 from urllib.parse import urlparse
 
-from .http_response import (create_authorization_http_headers, get_user_jwt_from_event,  # noqa - used for backwards compatibility
+from dotenv import load_dotenv
+from url_remote.environment_name_enum import EnvironmentName
+
+from .mini_logger import MiniLogger as logger
+from .http_response import (create_authorization_http_headers,  # noqa - used for backwards compatibility
+                            get_user_jwt_from_event,
                             create_return_http_headers, create_error_http_response, create_ok_http_response,
                             create_http_body)
-from .mini_logger import MiniLogger as logger
 
 load_dotenv()
+override = True if os.getenv("override", "").lower() in ("t", "true", "0") else False
+load_dotenv(dotenv_path=os.environ.get("DOTENV_PATH"), override=override)  # add another .env file if needed
+
 # raise Exception("Failed to load environment variables from .env file\n"
 #                 "Please check if the file exists, maybe you are not in the right venv?")
 
 # TODO: add cache with/out timeout decorator
 
+# TODO Use the const/enum from language package
 DEFAULT_LANG_CODE_STR = "en"
+# TODO If no one is using DEFAULT_DATETIME_FORMAT, let's comment it
+# TODO Lets split DEFAULT_DATETIME_FORMAT into two const 1. Displayable to Humans (MMM) 2. Internal (sortable) with %m
+DEFAULT_DATETIME_FORMAT = "%Y-%m-%d %H:%M:%S"
+
+
+def append_if_not_exist(lst: list, item: object) -> None:
+    if item not in lst:
+        lst.append(item)
+
+
+def to_dict(data: json or dict or None) -> dict:
+    if data is None:
+        return {}
+    if isinstance(data, dict):
+        return data
+    return json.loads(data)
+
+
+def to_json(data: json or dict or None) -> json:
+    if data is None:
+        data = {}
+    if isinstance(data, dict):
+        return json.dumps(data)
+    return data
+
 
 def timedelta_to_time_format(time_delta: timedelta) -> str:
     """
@@ -108,7 +144,7 @@ def is_valid_date_range(date_range: tuple) -> bool:
     return True
 
 
-def is_valid_datetime_range(datetime_range: tuple) -> bool:
+def is_valid_datetime_range(datetime_range: (datetime, datetime)) -> bool:
     """
     Validate that the datetime range is in the format 'YYYY-MM-DD HH:MM:SS'.
     """
@@ -117,10 +153,10 @@ def is_valid_datetime_range(datetime_range: tuple) -> bool:
         logger.end(object={"is_valid_datetime_range_result": False, "reason": "len(datetime_range) != 2"})
         return False
 
-    for datetime_obj in datetime_range:
-        if not isinstance(datetime_obj, datetime):
-            logger.end(object={"is_valid_datetime_range_result": False, "reason": "datetime_range contains non-datetime objects"})
-            return False
+    if not all(isinstance(datetime_obj, datetime) for datetime_obj in datetime_range):
+        logger.end(object={"is_valid_datetime_range_result": False,
+                           "reason": "datetime_range contains non-datetime objects"})
+        return False
     logger.end(object={"is_valid_datetime_range_result": True})
     return True
 
@@ -216,7 +252,7 @@ def is_date_in_date_range(check_date: date, date_range: tuple) -> bool:
     return start_date <= check_date <= end_date
 
 
-def is_datetime_in_datetime_range(check_datetime: datetime, datetime_range: tuple) -> bool:
+def is_datetime_in_datetime_range(check_datetime: datetime, datetime_range: (datetime, datetime)) -> bool:
     """
     Check if the given datetime is within the specified datetime range.
 
@@ -250,11 +286,14 @@ def get_environment_name() -> str:
     return environment_name
 
 
-def our_get_env(key: str, default: str = None, raise_if_not_found: bool = True) -> str:
-    logger.start(object={"key": key, "default": default, "raise_if_not_found": raise_if_not_found})
+def our_get_env(key: str, default: str = None, raise_if_not_found: bool = True,
+                raise_if_empty: bool = False) -> str:
+    logger.start(object={"key": key, "default": default, "raise_if_not_found": raise_if_not_found,
+                         "raise_if_empty": raise_if_empty})
     env_var = os.getenv(key, default)
-    if raise_if_not_found and env_var is None:
-        raise Exception(f"Environment variable {key} not found")
+    if ((raise_if_not_found and key not in os.environ and default is None)
+            or (raise_if_empty and not env_var and not default)):
+        raise Exception(f"Environment variable {key} not found - please check your .env file")
     logger.end(object={"env_var": env_var})
     return env_var
 
@@ -271,5 +310,105 @@ def get_sql_password() -> str:
     return our_get_env("RDS_PASSWORD")
 
 
+def get_dialog_jwt_secret_key() -> str:
+    return our_get_env("DIALOG_JWT_SECRET_KEY", raise_if_empty=True)
+
+
+def encode_jwt(payload: dict, key: str) -> str:
+    """Example:
+    payload = {
+        "user_id": 123,
+        "username": "john_doe",
+        "exp": datetime.datetime.utcnow() + datetime.timedelta(hours=1)
+    }
+    """
+    return jwt.encode(payload, key, algorithm='HS256')
+
+
+def decode_jwt(token: str, key: str) -> dict:
+    """key can be private / public"""
+    return jwt.decode(token, key, algorithms=['HS256'])
+
+
+def obfuscate_log_dict(log_dict: dict) -> dict:
+    """
+    Obfuscate keys:
+    - %password%
+    - %secret%
+    - %token%
+    - %jwt%
+    - %e%mail%
+    - %phone%
+    - %name% (first / last / nick / user etc.)
+    - %address%
+    - %ssn%
+    """
+    environment_name = get_environment_name()
+    if environment_name == EnvironmentName.PLAY1.value:
+        return log_dict
+    obfuscated_log_dict = {}
+    for key, value in log_dict.items():
+        if isinstance(value, dict):
+            obfuscated_log_dict[key] = obfuscate_log_dict(value)
+        elif re.search(r'password|secret|token|jwt|e[\-_]?mail|phone|name|address|ssn', key, re.IGNORECASE):
+           # TODO Maybe we can reveal a small part
+            obfuscated_log_dict[key] = "***"
+        else:
+            obfuscated_log_dict[key] = value
+    return obfuscated_log_dict
+
+# TODO: add tests to the following functions
+
 def remove_digits(text: str) -> str:
-    return ''.join([i for i in text if not i.isdigit()])
+    return ''.join(i for i in text if not i.isdigit())
+
+
+def generate_otp():
+    """Generates a 6-digit OTP"""
+    otp = random.randint(100000, 999999)
+    return otp
+
+
+def get_current_datetime_string(datetime_format: str = DEFAULT_DATETIME_FORMAT, tz: timezone = None) -> str:
+    current_datetime_string = datetime.now(tz).strftime(datetime_format)
+    return current_datetime_string
+
+
+def datetime_from_str(date_str: str, datetime_format: str = DEFAULT_DATETIME_FORMAT) -> datetime:
+    return datetime.strptime(date_str, datetime_format)
+
+
+def datetime_to_str(input_datetime: datetime, datetime_format: str = DEFAULT_DATETIME_FORMAT) -> str:
+    return input_datetime.strftime(datetime_format)
+
+
+def validate_arguments(args: dict) -> None:
+    """
+    Validate method arguments to ensure they are not None or ''
+    :param args: arguments to be validated (usually locals())
+    :return: True if all arguments are not None or '', False otherwise
+    """
+    for arg, value in args.items():
+        if not arg:
+            raise ValueError(f"Argument {arg} is cannot be empty (got {value})")
+
+
+def reformat_time_string(input_str: str) -> str:
+    """Example:
+    "1234" -> "12:34:00:00"
+    """
+    hours = input_str[:2]
+    minutes = "00" if len(input_str) < 4 else input_str[2:4]
+    seconds = "00" if len(input_str) < 6 else input_str[4:6]
+    milliseconds = "00" if len(input_str) < 8 else input_str[6:8]
+    # TODO Shall we use DEFAULT_DATETIME_FORMAT and have the formats in central 
+    time_format = f"{hours}:{minutes}:{seconds}:{milliseconds}"
+    return time_format
+
+
+def get_ip_v4():
+    return requests.get('https://ipv4.seeip.org/jsonip').json()['ip']
+
+
+def get_ip_v6():
+    return requests.get('https://api.seeip.org/jsonip').json()['ip']
